import bluebelt.helpers.ticks
import matplotlib.pyplot as plt
import os

def plot_defaults(ax, fig, xlim=(None, None), ylim=(None, None), max_xticks=None, format_xticks=None, format_yticks=None, title=None, xlabel=None, ylabel=None, legend=True, path=None, **kwargs):

    # # limit axis
    # xlim = kwargs.pop("xlim", (None, None))
    # ylim = kwargs.pop('ylim', (None, None))
    
    # # set ticks
    # max_xticks = kwargs.pop('max_xticks', None)
    
    # # format ticks
    # format_xticks = kwargs.pop('format_yticks', None) # "1.2f", "1.1%"
    # format_yticks = kwargs.pop('format_yticks', None)
    

    # # labels
    # title = kwargs.pop('title', None)
    # ylabel = kwargs.pop('ylabel', None)
    # xlabel = kwargs.pop('xlabel', None)
        
    # # legend
    # legend = kwargs.pop('legend', True)
    
    # # file
    # path = kwargs.pop('path', None)


    #
    #
    #
    
    # histogram
    bins = kwargs.pop('bins', 20)
    dist = kwargs.pop("dist", "norm")
    
    # pattern
    bounds = kwargs.pop('bounds', True)
    
    # equals means / variances
    digits = kwargs.pop("digits", 3)


    #
    #
    #

    # limit axis
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    
    # set ticks
    if max_xticks is None:
        max_xticks = bluebelt.helpers.ticks.get_max_xticks(ax)
    bluebelt.helpers.ticks.year_week(_poly.result, ax=ax, max_xticks=max_xticks)

    # format ticks
    if format_xticks:
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels([f'{x:{format_xticks}}' for x in ax.get_xticks()])
    if format_yticks:
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels([f'{y:{format_yticks}}' for y in ax.get_yticks()])

    # labels
    if title:
        ax.set_title(title)
    if xlabel:
        ax.set_xlabel(xlabel)
    if ylabel:
        ax.set_ylabel(ylabel)

    # legend
    if legend:
        ax.legend()
    elif ax.get_legend() is not None:
        ax.get_legend().set_visible(False)
    
    plt.tight_layout()

    # file
    if path:
        if len(os.path.dirname(path)) > 0 and not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        plt.savefig(path)
        plt.close()
    else:
        plt.close()
        return fig
    



import os
import bluebelt
import pandas as pd

import bluebelt.helpers.ticks
import bluebelt.data.resolution
import matplotlib.pyplot as plt

from bluebelt.helpers.decorators.performance import performance

# to do
# -----
# add line plot for volume % per weekday per week
# add line plot for skill volume % per week and/or date

@performance
class Effort():
    def __init__(self, _obj, skip_na=True, skip_zero=True):  
        self._obj = _obj
        self.quantity = bluebelt.data.resolution.Resample(self._obj, 7).diff_quantity(skip_na=skip_na, skip_zero=skip_zero)
        self.distribution = bluebelt.data.resolution.Resample(self._obj, 7).diff_distribution(skip_na=skip_na, skip_zero=skip_zero)
        self.skills = bluebelt.data.resolution.Resample(self._obj, 7).diff_skills(skip_na=skip_na, skip_zero=skip_zero)
        self.qds = 1 - ((1 - self.quantity) * (1 - self.distribution) * (1 - self.skills))

    def __repr__(self):
        return (f'{self.__class__.__name__}(n={self._obj.shape[0]:1.0f}, qds={self.qds.mean():1.4f}, quantity={self.quantity.mean():1.4f}, distribution={self.distribution.mean():1.4f}, skills={self.skills.mean():1.4f})')
    
    def plot(self, xlim=(None, None), ylim=(0, 1), max_xticks=None, format_xticks=None, format_yticks=".0%", title=None, xlabel=None, ylabel=None, legend=True, path=None, **kwargs):
       
        # prepare figure
        fig, ax = plt.subplots(nrows=1, ncols=1, **kwargs)

        # q
        ax.fill_between(self.quantity.index[1:], 0, self.quantity.values[1:], label=f'quantity ({self.quantity.mean()*100:1.1f}%)', **bluebelt.config('effort.quantity.fill'))
        ax.plot(self.quantity.index[1:], self.quantity.values[1:], **bluebelt.config('effort.quantity.line'))
        
        # d
        ax.fill_between(self.distribution.index[1:], 0, self.distribution.values[1:], label=f'distribution ({self.distribution.mean()*100:1.1f}%)', **bluebelt.config('effort.distribution.fill'))
        ax.plot(self.distribution.index[1:], self.distribution.values[1:], **bluebelt.config('effort.distribution.line'))

        # s
        ax.fill_between(self.skills.index[1:], 0, self.skills.values[1:], label=f'skills ({self.skills.mean()*100:1.1f}%)', **bluebelt.config('effort.skills.fill'))
        ax.plot(self.skills.index[1:], self.skills.values[1:], **bluebelt.config('effort.skills.line'))
        
        # qds
        ax.plot(self.qds.index[1:], self.qds.values[1:], label=f'qds effort ({self.qds.mean()*100:1.1f}%)', **bluebelt.config('effort.qds.line'))
        
        # limit axis
        ax.set_xlim(xlim)
        ax.set_ylim(ylim)
        
        # set xticks
        if max_xticks is None:
            max_xticks = bluebelt.helpers.ticks.get_max_xticks(ax)
        bluebelt.helpers.ticks.year_week(self.qds, ax=ax, max_xticks=max_xticks)

        # format ticks
        if format_xticks:
            ax.set_xticks(ax.get_xticks())
            ax.set_xticklabels([f'{x:{format_xticks}}' for x in ax.get_xticks()])
        if format_yticks:
            ax.set_yticks(ax.get_yticks())
            ax.set_yticklabels([f'{y:{format_yticks}}' for y in ax.get_yticks()])

        # labels
        if title:
            ax.set_title(title)
        if xlabel:
            ax.set_xlabel(xlabel)
        if ylabel:
            ax.set_ylabel(ylabel)

        # legend
        if legend:
            ax.legend()
        elif ax.get_legend() is not None:
            ax.get_legend().set_visible(False)
        
        plt.tight_layout()

        # file
        if path:
            if len(os.path.dirname(path)) > 0 and not os.path.exists(os.path.dirname(path)):
                os.makedirs(os.path.dirname(path))
            plt.savefig(path)
            plt.close()
        else:
            plt.close()
            return fig

@performance
class Ease():
    def __init__(self, _obj, skip_na=True, skip_zero=True):
        self._obj = _obj
        self.quantity = 1 - bluebelt.data.resolution.Resample(self._obj, 7).diff_quantity(skip_na=skip_na, skip_zero=skip_zero)
        self.distribution = 1 - bluebelt.data.resolution.Resample(self._obj, 7).diff_distribution(skip_na=skip_na, skip_zero=skip_zero)
        self.skills = 1 - bluebelt.data.resolution.Resample(self._obj, 7).diff_skills(skip_na=skip_na, skip_zero=skip_zero)
        self.qds = self.quantity * self.distribution * self.skills

    def __repr__(self):
        return (f'{self.__class__.__name__}(n={self._obj.shape[0]:1.0f}, qds={self.qds.mean():1.4f}, quantity={self.quantity.mean():1.4f}, distribution={self.distribution.mean():1.4f}, skills={self.skills.mean():1.4f})')
    
    def plot(self, xlim=(None, None), width=0.8, format_yticks=".1%", title=None, xlabel=None, ylabel=None, path=None, **kwargs):
            
        title = title or f'planning QDS ease plot'
        
        # calculations
        qds = pd.Series({
            'q': self.quantity.mean(),
            'd': self.distribution.mean(),
            's': self.skills.mean(),
        }).reset_index(drop=True)

        vals = qds.cumprod()
        rest = (qds.cumprod().shift(1).fillna(1) - qds.cumprod())

        # prepare figure
        fig, ax = plt.subplots(nrows=1, ncols=1, **kwargs)

        # top
        ax.bar(rest.index, rest.values, bottom=vals.values, width=width, **bluebelt.config('ease.top.bar'))
        ax.bar(rest.index, rest.values, bottom=vals.values, width=width, **bluebelt.config('ease.borders.bar'))
        
        # bottom
        ax.bar(vals.index, vals.values, width=width, **bluebelt.config('ease.bottom.bar'))
        ax.bar(vals.index, vals.values, width=width, **bluebelt.config('ease.borders.bar'))
        
        
        # connectors
        xlim = ax.get_xlim()
        ax.bar((rest.index+0.5), 0, bottom=vals.values, width=1-width, **bluebelt.config('ease.connectors.bar'))
        
        ax.set_ylim(0,1.1)
        ax.set_xlim(xlim)
        ax.set_xticks([0,1,2])
        ax.set_xticklabels(['Q','D','S'])
        ax.tick_params(axis="x", bottom=True, top=False, labelbottom=True, labeltop=False)

        for tick in ax.get_xticks():
            ax.text(tick, vals.iloc[tick]+(rest.iloc[tick] / 2), f'{(1-qds.iloc[tick]):{format_yticks}}', **bluebelt.config('ease.values'))
            ax.text(tick, (vals.iloc[tick] / 2), f'{qds.iloc[tick]:{format_yticks}}', **bluebelt.config('ease.values'))
            
        # manage yticks
        yticks = [0, qds.cumprod().iloc[-1], 1]
        ax.set_yticks(yticks)
        ax.set_yticklabels([f'{y:{format_yticks}}' for y in ax.get_yticks()])
        ax.yaxis.tick_right()

        # labels
        if title:
            ax.set_title(title)
        if xlabel:
            ax.set_xlabel(xlabel)
        if ylabel:
            ax.set_ylabel(ylabel)

        plt.tight_layout()

        # file
        if path:
            if len(os.path.dirname(path)) > 0 and not os.path.exists(os.path.dirname(path)):
                os.makedirs(os.path.dirname(path))
            plt.savefig(path)
            plt.close()
        else:
            plt.close()
            return fig
        


import numpy as np
import pandas as pd

import bluebelt.helpers.date

import datetime


def project(_obj, year=None, adjust_holidays=True):

    _obj = _obj.copy()

    # check if _obj has only one iso year
    if _obj.index.isocalendar().year.unique().size > 1:
        raise ValueError("The _obj can only contain one iso year")

    # check for week 53
    if (
        _obj.index.isocalendar().week.max()
        > bluebelt.helpers.date.last_iso_week_in_year(year)
    ):
        # drop week 53
        _obj = _obj[_obj.index.isocalendar().week < 53]
    elif (
        _obj.index.isocalendar().week.max()
        < bluebelt.helpers.date.last_iso_week_in_year(year)
    ):
        # create week 53
        pass

    # create a isoformat MultiIndex
    index = pd.MultiIndex.from_frame(_obj.index.isocalendar())
    # set the year to the requested year
    index = index.set_levels([year], level=0)
    # build a new DatetimeIndex from the isoformat MultiIndex
    index = pd.DatetimeIndex(
        np.apply_along_axis(
            lambda x: datetime.datetime.fromisocalendar(x[0], x[1], x[2]),
            1,
            np.array([*index.values]),
        )
    )
    # create the new _obj
    if isinstance(_obj, pd.Series):
        result = pd.Series(index=index, data=_obj.values, name=_obj.name)
    else:
        result = pd.DataFrame(index=index, data=_obj.values, columns=_obj.columns)

    if adjust_holidays:

        # get original holidays
        _obj_holidays = bluebelt.helpers.holidays.get_holidays_dict(
            _obj.index.year.unique()
        )
        # find the new dates for the original holidays
        _obj_holidays = {
            datetime.datetime.fromisocalendar(
                year, key.isocalendar()[1], key.isocalendar()[2]
            ): value
            for key, value in _obj_holidays.items()
            if key in _obj.index
        }

        # get new holidays
        result_holidays = bluebelt.helpers.holidays.get_holidays_dict(
            result.index.year.unique()
        )
        result_holidays = {
            value: key for key, value in result_holidays.items() if key in result.index
        }

        # create a swap dictionairy
        holidays = {
            key: result_holidays.get(value) for key, value in _obj_holidays.items()
        }

        for key, value in holidays.items():

            # catch none existing holidays like liberation day in some years
            if key and value:
                result.loc[[key, value]] = result.loc[[value, key]].values

    return result



import os

import bluebelt
import pandas as pd
import numpy as np

import matplotlib.pyplot as plt

import bluebelt.helpers.ticks

import warnings

def _get_frame(_obj, **kwargs):

    columns = kwargs.get('columns', None)

    if isinstance(_obj, pd.DataFrame) and isinstance(columns, (str, list)):
        return _obj[columns]
    elif isinstance(_obj, pd.Series):
        return pd.DataFrame(_obj)
    else:
        return _obj

def _get_name(_obj, **kwargs):

    if isinstance(_obj, pd.Series):
        return _obj.name
    elif isinstance(_obj, pd.DataFrame):
        names = []
        for col in _obj.columns:
            names.append(col)
        return bluebelt.core.helpers._get_nice_list(names)
    else:
        return None

def line(_obj, xlim=(None, None), ylim=(None, None), max_xticks=None, format_xticks=None, format_yticks=None, title=None, xlabel=None, ylabel=None, legend=True, path=None, **kwargs):
    frame = _get_frame(_obj, **kwargs)

    # prepare figure
    fig, ax = plt.subplots(nrows=1, ncols=1, **kwargs)

    for id, col in enumerate(frame):
        ax.plot(frame[col].index, frame[col].values, **bluebelt.config(f"line{id%6}"), label=col)
        
    # limit axis
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    
    # set ticks
    if max_xticks is None:
        max_xticks = bluebelt.helpers.ticks.get_max_xticks(ax)
    bluebelt.helpers.ticks.year_week(frame, ax=ax, max_xticks=max_xticks)

    # format ticks
    if format_xticks:
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels([f'{x:{format_xticks}}' for x in ax.get_xticks()])
    if format_yticks:
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels([f'{y:{format_yticks}}' for y in ax.get_yticks()])

    # labels
    if title:
        ax.set_title(title)
    if xlabel:
        ax.set_xlabel(xlabel)
    if ylabel:
        ax.set_ylabel(ylabel)

    # legend
    if legend:
        ax.legend()
    elif ax.get_legend() is not None:
        ax.get_legend().set_visible(False)
    
    plt.tight_layout()

    # file
    if path:
        if len(os.path.dirname(path)) > 0 and not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        plt.savefig(path)
        plt.close()
    else:
        plt.close()
        return fig

def area(_obj, xlim=(None, None), ylim=(None, None), max_xticks=None, format_xticks=None, format_yticks=None, title=None, xlabel=None, ylabel=None, legend=True, path=None, **kwargs):
    frame = _get_frame(_obj, **kwargs)

    # prepare figure
    fig, ax = plt.subplots(nrows=1, ncols=1, **kwargs)

    for id, col in enumerate(frame):
        ax.fill_between(frame[col].index, frame[col].values, **bluebelt.config(f"fill{id%6}"))
        ax.plot(frame[col].index, frame[col].values, **bluebelt.config(f"line{id%6}"), label=col)
        
    # limit axis
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    
    # set ticks
    if max_xticks is None:
        max_xticks = bluebelt.helpers.ticks.get_max_xticks(ax)
    bluebelt.helpers.ticks.year_week(frame, ax=ax, max_xticks=max_xticks)

    # format ticks
    if format_xticks:
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels([f'{x:{format_xticks}}' for x in ax.get_xticks()])
    if format_yticks:
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels([f'{y:{format_yticks}}' for y in ax.get_yticks()])

    # labels
    if title:
        ax.set_title(title)
    if xlabel:
        ax.set_xlabel(xlabel)
    if ylabel:
        ax.set_ylabel(ylabel)

    # legend
    if legend:
        ax.legend()
    elif ax.get_legend() is not None:
        ax.get_legend().set_visible(False)
    
    plt.tight_layout()

    # file
    if path:
        if len(os.path.dirname(path)) > 0 and not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        plt.savefig(path)
        plt.close()
    else:
        plt.close()
        return fig

def hist(_obj, bins=20, xlim=(None, None), ylim=(None, None), format_xticks=None, format_yticks=None, title=None, xlabel=None, ylabel=None, legend=True, path=None, **kwargs):
    frame = _get_frame(_obj, **kwargs)

    # prepare figure
    fig, ax = plt.subplots(nrows=1, ncols=1, **kwargs)

    bins = (np.nanmax(frame.to_numpy())-np.nanmin(frame.to_numpy()))/bins
    bins = np.arange(np.nanmin(frame), np.nanmax(frame)+bins, bins)

    for id, col in enumerate(frame):
        ax.hist(frame[col], bins=bins, label=col, **bluebelt.config(f"hist{id%6}"))
        
    # limit axis
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    
    # format ticks
    if format_xticks:
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels([f'{x:{format_xticks}}' for x in ax.get_xticks()])
    if format_yticks:
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels([f'{y:{format_yticks}}' for y in ax.get_yticks()])

    # labels
    if title:
        ax.set_title(title)
    if xlabel:
        ax.set_xlabel(xlabel)
    if ylabel:
        ax.set_ylabel(ylabel)

    # legend
    if legend:
        ax.legend()
    elif ax.get_legend() is not None:
        ax.get_legend().set_visible(False)
    
    plt.tight_layout()

    # file
    if path:
        if len(os.path.dirname(path)) > 0 and not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        plt.savefig(path)
        plt.close()
    else:
        plt.close()
        return fig

def box(_obj, xlim=(None, None), ylim=(None, None), format_xticks=None, format_yticks=None, title=None, xlabel=None, ylabel=None, path=None, **kwargs):
    frame = _get_frame(_obj, **kwargs)
    labels = frame.columns

    # drop na values
    frame = [series.dropna().to_list() for name, series in frame.iteritems()]

    # prepare figure
    fig, ax = plt.subplots(nrows=1, ncols=1, **kwargs)

    boxplot = ax.boxplot(frame, labels=labels, **kwargs)

    for box in boxplot['boxes']:
        box.set(**bluebelt.config('boxplot.boxes'))
        
    # limit axis
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    
    # format ticks
    if format_xticks:
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels([f'{x:{format_xticks}}' for x in ax.get_xticks()])
    if format_yticks:
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels([f'{y:{format_yticks}}' for y in ax.get_yticks()])

    # labels
    if title:
        ax.set_title(title)
    if xlabel:
        ax.set_xlabel(xlabel)
    if ylabel:
        ax.set_ylabel(ylabel)

    plt.tight_layout()

    # file
    if path:
        if len(os.path.dirname(path)) > 0 and not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        plt.savefig(path)
        plt.close()
    else:
        plt.close()
        return fig

def waterfall(series, horizontal=False, invertx=False, inverty=False, width=0.6, height=0.6, xlim=(None, None), ylim=(None, None), format_xticks=None, format_yticks=None, title=None, xlabel=None, ylabel=None, path=None, **kwargs):

    title = title or f"{_get_name(series)} waterfall"

    if not isinstance(series, pd.Series):
        raise ValueError('Waterfall charts need a pandas.Series')
    
    measure = pd.Series(kwargs.pop('measure', ['relative'] * series.shape[0]))

    # are the totals ok?
    if ('total' in measure.unique()) and not (series.where((measure=='relative').values).cumsum().shift().where((measure=='total').values).fillna(0) == series.where((measure=='total').values).fillna(0)).all():
        warnings.warn('The totals values are not the totals of the preceeding values. This will be adjusted.', Warning)
        series = series.where((measure=='relative').values).cumsum().shift().where((measure=='total').values, series).fillna(0)

    # calculations
    bottom = series.where((measure=='relative').values).fillna(0).cumsum() - series
    index = np.arange(series.index.shape[0])

    ylim = ylim or ((bottom).min() * 1.05, (series+bottom).max() * 1.05)
    xlim = xlim or ((bottom).min() * 1.05, (series+bottom).max() * 1.05)
    
    fig, ax = plt.subplots(nrows=1, ncols=1)
    
    if horizontal:
        
        # totals
        ax.barh(index, series.where((measure=='total').values).values, left=bottom, height=height, **bluebelt.config('waterfall.total'))
        ax.barh(index, series.where((measure=='total').values).values, left=bottom, height=height, **bluebelt.config('waterfall.border'))

        # increasing
        ax.barh(index, series.where((series>=0) & ((measure=='relative').values)).values, left=bottom, height=height, **bluebelt.config('waterfall.increasing'))
        ax.barh(index, series.where((series>=0) & ((measure=='relative').values)).values, left=bottom, height=height, **bluebelt.config('waterfall.border'))

        # decreasing
        ax.barh(index, series.where((series<0) & ((measure=='relative').values)).values, left=bottom, height=height, **bluebelt.config('waterfall.decreasing'))
        ax.barh(index, series.where((series<0) & ((measure=='relative').values)).values, left=bottom, height=height, **bluebelt.config('waterfall.border'))

        # connectors
        ax.vlines(x=(bottom + series)[:-1], ymin=(index)[:-1], ymax=(index+0.5)[:-1]+(1-height), **bluebelt.config('waterfall.connectors'))

        # yticks
        ax.set_yticks(index)
        ax.set_yticklabels(series.index.values)
        
        # swap margins
        xmargin, ymargin = ax.margins()        
        ax.set_xmargin(ymargin)
        ax.set_ymargin(xmargin)

    else:
        # totals
        ax.bar(index, series.where((measure=='total').values).values, bottom=bottom, width=width, **bluebelt.config('waterfall.total'))
        ax.bar(index, series.where((measure=='total').values).values, bottom=bottom, width=width, **bluebelt.config('waterfall.border'))

        # increasing
        ax.bar(index, series.where((series>=0) & ((measure=='relative').values)).values, bottom=bottom, width=width, **bluebelt.config('waterfall.increasing'))
        ax.bar(index, series.where((series>=0) & ((measure=='relative').values)).values, bottom=bottom, width=width, **bluebelt.config('waterfall.border'))

        # decreasing
        ax.bar(index, series.where((series<0) & ((measure=='relative').values)).values, bottom=bottom, width=width, **bluebelt.config('waterfall.decreasing'))
        ax.bar(index, series.where((series<0) & ((measure=='relative').values)).values, bottom=bottom, width=width, **bluebelt.config('waterfall.border'))

        # connectors
        ax.hlines(y=(bottom + series)[:-1], xmin=(index)[:-1], xmax=(index+0.5)[:-1]+(1-height), **bluebelt.config('waterfall.connectors'))

        # xticks
        ax.set_xticks(index)
        ax.set_xticklabels(series.index.values)
        
    # limit axis
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    
    # invert axis
    if invertx:
        ax.invert_xaxis()
    if inverty:
        ax.invert_yaxis()
    
    # format ticks
    if format_xticks:
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels([f'{x:{format_xticks}}' for x in ax.get_xticks()])
    if format_yticks:
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels([f'{y:{format_yticks}}' for y in ax.get_yticks()])

    # labels
    if title:
        ax.set_title(title)
    if xlabel:
        ax.set_xlabel(xlabel)
    if ylabel:
        ax.set_ylabel(ylabel)
    
    plt.tight_layout()

    # file
    if path:
        if len(os.path.dirname(path)) > 0 and not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        plt.savefig(path)
        plt.close()
    else:
        plt.close()
        return fig
    



# from bluebelt.core import index
from bluebelt.core import series
from bluebelt.core import dataframe

from bluebelt.create import create

import bluebelt.styles.rc

import os
from collections.abc import MutableMapping

import matplotlib as mpl
import yaml


class BlueConfig:
    _config = {}

    @staticmethod
    def get(name):
        if name in BlueConfig._config.keys():
            return BlueConfig._config[name]
        elif (
            ".".join(name.split(".")[:-1]) in BlueConfig._config.keys()
        ):  # is the name up to the last dot in keys?
            # is the last part of the name in this valueset?
            if (
                name.split(".")[-1]
                in BlueConfig._config[".".join(name.split(".")[:-1])]
            ):
                return BlueConfig._config[".".join(name.split(".")[:-1])][
                    name.split(".")[-1]
                ]
            else:
                return None
        else:
            return None

    @staticmethod
    def set(name, value):
        BlueConfig._config[name] = value

    @staticmethod
    def default():
        set_style("paper")


def config(key=None, value=None):
    """
    Change or get the Bluebelt configuration.

    Parameters
    ----------
    key: the name of the configuration parameter
    value: the value of the configuration parameter

    Returns
    -------
    If key and value are provided parameter key is set to value and nothing is
    returned.
    If only key is provided the parameter value is returned.
    If key and value are both not provided the configuration is reset to the
    default parameter values.
    """

    if key and value:
        BlueConfig.set(key, value)
    elif key:
        return BlueConfig.get(key)
    else:
        BlueConfig.default()


def set_style(name=None):
    """
    Set the default style

    Parameters
    ----------
    name : string, default None
        a string with a default bluebelt style name or the path to a custuom
        YAML style sheet

    Returns
    -------
    None

    Example
    -------
    bluebelt.set_style("paper")

    """
    if name is None:
        return BlueConfig.get("style")

    path = os.path.dirname(os.path.realpath(__file__))

    # check if name is not pointing to a yaml file
    if os.path.splitext(name)[1] not in [".yaml", ".yml"]:
        path_file = f"{path}/styles/{name}.yaml"

        # check if the style exists
        if not os.path.isfile(path_file):

            # list styles
            style_list = ""
            for file in os.listdir(f"{path}/styles/"):
                if file.endswith(".yaml"):
                    if len(style_list) > 0:
                        style_list += ", "
                    style_list += str(file.split(".")[0])
            raise ValueError(
                f"{name} style does not exist. Choose from {str(style_list)}"
            )
    else:
        path_file = name

    try:
        with open(path_file, "r") as file:
            style = _flatten_style(yaml.load(file, yaml.SafeLoader))

        for key, value in style.items():
            if key.startswith("rc."):
                # set the values in matplotlib and bluebelt
                mpl.rc(key.split(".")[-1], **value)
                BlueConfig.set(key, value)
            else:
                BlueConfig.set(key, value)

    except OSError as e:
        print(f"Unable to find {path_file}. Did you enter the correct file path?")


def get_style(name):
    """
    Return the contents of a bluebelt style

    Parameters
    ----------
    name : string, default None
        a string with a default bluebelt style name or the path to a custuom
        YAML style sheet

    Returns
    -------
    bluebelt style dict

    Example
    -------
    bluebelt.set_style("paper")

    """
    path = os.path.dirname(os.path.realpath(__file__))

    # check if name is not pointing to a yaml file
    if os.path.splitext(name)[1] not in [".yaml", ".yml"]:
        name = f"{path}/styles/{name}.yaml"

    with open(name, "r") as file:
        style = _flatten_style(yaml.load(file, yaml.SafeLoader))

    return style


def _flatten_style(
    d: MutableMapping, parent_key: str = "", sep: str = "."
) -> MutableMapping:
    items = []
    for k, v in d.items():
        new_key = parent_key + sep + k if parent_key else k
        if isinstance(v, MutableMapping):
            if any(map(lambda x: isinstance(x, dict), v.values())):
                items.extend(_flatten_style(v, new_key, sep=sep).items())
            else:
                items.append((new_key, v))
        else:
            items.append((new_key, v))
    return dict(items)


def set_figsize_in_pixels(
    size: tuple or int = None, height: int = None, dpi: int = None
):
    """
    Set the default figsize in pixels

    Parameters
    ----------
    size : tuple or int, default None
        a tuple with (width, height) values or
        an int with the 'width' value in which case a height must be provided
    height: int, default None
        if size is an int height will complete the figsize
    dpi: int, default None
        change the default dpi (dots per inch, or dots per 2.54 cm)

    Returns
    -------
    None

    Example
    -------
    bluebelt.set_figsize_in_pixels(900, 600)

    """
    # set or get dpi
    if dpi:
        mpl.rcParams["figure.dpi"] = dpi
    else:
        dpi = mpl.rcParams["figure.dpi"]

    # in case two int are passed
    if height and isinstance(size, int):
        size = (size, height)

    # set figsize
    mpl.rcParams["figure.figsize"] = tuple(ti / dpi for ti in size)


def set_figsize_in_cm(size: tuple or int = None, height: int = None, dpi: int = None):
    """
    Set the default figsize in centimeters

    Parameters
    ----------
    size : tuple or int, default None
        a tuple with (width, height) values or
        an int with the 'width' value in which case a height must be provided
    height: int, default None
        if size is an int height will complete the figsize
    dpi: int, default None
        change the default dpi (dots per inch, or dots per 2.54 cm)

    Returns
    -------
    None

    Example
    -------
    bluebelt.set_figsize_in_cm(5, 8)

    """
    # set or get dpi
    if dpi:
        mpl.rcParams["figure.dpi"] = dpi
    else:
        dpi = mpl.rcParams["figure.dpi"]

    # in case two int are passed
    if height and isinstance(size, int):
        size = (size, height)

    # set figsize
    mpl.rcParams["figure.figsize"] = tuple(ti / 2.54 for ti in size)


def set_transparent(transparent: bool = False):
    """
    Set the default way to handle transparency when saving an image.

    Parameters
    ----------
    transparent : bool, default False

    Returns
    -------
    None

    Example
    -------
    bluebelt.set_transparent(True)

    """
    mpl.rcParams["savefig.transparent"] = transparent


def set_language(language=None):
    """
    Set the default language.

    Parameters
    ----------
    language : str, default None
        currently only support 'en' and 'nl'
        currently only works for weekday names

    Returns
    -------
    None

    Example
    -------
    bluebelt.set_language('en')

    """
    languages = ["nl", "en"]
    if language not in languages:
        raise ValueError(
            f"The language should be one of {str(languages)[1:-1]}, not {language}."
        )

    BlueConfig.set("language", language)


def get_language():
    """
    Get the current language.

    Returns
    -------
    str

    Example
    -------
    bluebelt.get_language()

    """
    return BlueConfig.get("language")


set_style("paper")
set_language("en")
